from django.shortcuts import get_object_or_404
from django.views.decorators.http import (
    require_POST,
    )
from revtracker.api_helpers import (
    ajax_response,
    check_user_allowed,
    )
from revtracker.templating import (
    markdownize_for_editor,
    )

from . import data
from .forms import StepSetDescriptionForm
from .models import Step




@require_POST
@check_user_allowed
def api_v1_step_set_description(request, step_id=None):
    step = get_object_or_404(Step, pk=step_id)
    errors = dict()
    set_description_form = StepSetDescriptionForm(request.POST)
    if set_description_form.is_valid():
        description = set_description_form.cleaned_data['description']
        step.description = description
        step.save()
        info = data.step_info(step)
        markdownize_for_editor(info, 'description')
        return ajax_response({'status': 'ok', 'description': info['description']})
    else:
        return ajax_response({'status': 'failed', 'errors': set_description_form.errors})
