from django import forms
from django.forms.widgets import TextInput

from . import models


class NumberInput(TextInput):
    input_type = 'number'


class ProjectUpdateRevisionForm(forms.Form):
    # Let's hope no-one wants to deploy a non-mainline revision
    revno = forms.IntegerField(error_messages={"invalid": "Invalid revision number"},
            widget=NumberInput)


class ProjectSetDescriptionForm(forms.ModelForm):
    class Meta:
        model = models.Project
        fields = ('description',)


class RevisionChangeStatusForm(forms.ModelForm):
    class Meta:
        model = models.Revision
        fields = ('status',)


class RevisionChoiceField(forms.ModelChoiceField):

    def label_from_instance(self, obj):
        return obj.title


class SetRollbackForm(forms.ModelForm):

    rolled_back_by = RevisionChoiceField(queryset=models.Revision.objects.all(),
            required=False, empty_label="(none)")

    class Meta:
        model = models.Revision
        fields = ('rolled_back_by',)
