YUI.add("isitdeployable-revision", function (Y) {

    var TEMPLATE_NAME   = 'revision_form';
    var template = Y.isitdeployable.Templating.compile_template(TEMPLATE_NAME);

    function update_children(holder, revisions, refresh_url) {
        if (revisions.length < 1) {
            holder.removeAll();
            return
        }
        var sourceList = revisions.slice();
        var before_index = 0;
        while (sourceList.length > 0 || before_index < holder.size()) {
            var current_revno = holder.getRevno(before_index);
            var sourceElem = sourceList[0];
            function remove() {
                // Removed revisions
                holder.remove(before_index);
            }
            function add() {
                // New revisions
                var child = new Y.isitdeployable.Revision({
                    data: sourceElem,
                    refresh_url: refresh_url
                });
                holder.add(child, before_index);
                if (sourceElem.child_revisions) {
                    update_children(child, sourceElem.child_revisions, refresh_url);
                }
                sourceList.splice(0, 1);
                before_index++;
            }
            if (before_index >= holder.size()) {
                add();
            } else if (sourceList.length <= 0) {
                remove();
            } else if (current_revno > sourceElem.revno) {
                add()
            } else if (current_revno == sourceElem.revno) {
                // Altered revision
                holder.item(before_index).set('data', sourceElem);
                before_index++;
                sourceList.splice(0, 1);
            } else {
                remove()
            }
        }
    }

    Y.namespace("isitdeployable").RevisionList = Y.Base.create('revision_list', Y.Widget, [Y.WidgetParent], {
        renderUI: function() {
            var box = this.get('boundingBox');
            box.plug(Y.Plugin.NodeFocusManager, {
                // CSS Selector indicating the descendant Nodes whose focus to manage
                descendants: '.revision',

                //  Move focus the buttons in the toolbar by pressing the left and
                //  right arrow keys
                keys: {
                    // down,j
                    next: 'down:40,74',
                    // up,k
                    previous: 'down:38,75'
                },

                circular: false
            });
        },
        bindUI: function() {
            holder = this;
            this.after('revision:revision_status_updated', function(e) {
                holder.updateRevisions(e.data[this.get('response_revision_list_attr')]);
            });
        },
        updateRevisions: function(revisions) {
            var box = this.get('boundingBox');
            var focused = box.focusManager.get('focused');
            var focusedIndex;
            var refresh_url = this.get('refresh_url');
            if (focused) {
                focusedIndex = box.focusManager.get('activeDescendant');
            }
            update_children(this, revisions, refresh_url);
            box.focusManager.refresh();
            if (focused) {
                box.focusManager.focus(focusedIndex);
            }
        },

        getRevno: function(index) {
            var item = this.item(index);
            if (item) {
                return item.get('data').revno;
            }
        },

        grabFocus: function() {
            this.get('boundingBox').focusManager.focus();
        },
    }, {
        ATTRS: {
            defaultChildType: 'Revision',
            response_revision_list_attr: {
                value: 'undeployed_revisions'
            },
            refresh_url: {
                value: ''
            }
        },
    });

    Y.namespace("isitdeployable").Revision = Y.Base.create('revision', Y.Widget, [Y.WidgetParent, Y.WidgetChild], {
        getHtml: function() {
            var data = this.get('data');
            var data_without_children = Y.clone(data, true);
            data_without_children.child_revisions = [];
            var comment_input = this.get('contentBox').one('input[name=comment]');
            var current_comment = comment_input ? comment_input.get('value') : null;
            data_without_children.comment_form.current_text = current_comment;
            return template(data_without_children);
        },
        renderUI: function() {
            var contentBox = this.get("contentBox");
            if (!contentBox.one(".revision")) {
                var html = this.getHtml();
                contentBox.setHTML(html);
            }
            this._childrenContainer = contentBox.one('.revision');
        },
        rerender: function(e) {
            if (e.target == this) {
                var contentBox = this.get("contentBox");
                var child_nodes = [];
                this.each(function(child, index) {
                    var child_box = child.get('boundingBox');
                    child_nodes.push(child_box);
                    child_box.remove();
                });
                var html = this.getHtml();
                contentBox.setHTML(html);
                this._childrenContainer = contentBox.one('.revision');
                var holder = this;
                Y.Array.each(child_nodes, function(child, index) {
                    holder._childrenContainer.append(child);
                });
            }
        },
        updateChildren: function(e) {
            if (e.target == this) {
                var data = this.get('data');
                var revisions = [];
                var refresh_url = this.get('refresh_url');
                if (data.child_revisions) {
                    revisions = data.child_revisions;
                }
                update_children(this, revisions, refresh_url);
            }
        },
        getRevno: function(index) {
            var item = this.item(index);
            if (item) {
                return item.get('data').revno;
            }
        },
        bindUI: function() {
            this.after("dataChange", this.updateChildren);
            this.after("dataChange", this.rerender);
            var holder = this;
            this.get('boundingBox').delegate("submit", function(e) {
                e.halt();
                var data = holder.get('data');
                var url = '/api/v1/project/' + data.project.name + '/+revision-by-pk/' + data.pk + '/+change-status';
                holder._set_loading();
                Y.io(url, holder._get_io_cfg(this));
            }, '.deployability');
            this.get('boundingBox').delegate("submit", function(e) {
                e.preventDefault();
                // FIXME: need to port to the API style
                Y.io(this.get('action'), holder._get_io_cfg(this));
            }, '.rollback');
            this.get('boundingBox').delegate("submit", function(e) {
                if (e.target.test("form")) {
                    e.preventDefault();
                    Y.io(this.get('action'), holder._get_comment_io_cfg(this));
                }
            }, '.add_comment');
            // c -> focus comment box
            this.get('boundingBox').delegate("key", this._focus_comment_box, 'down:67', '.revision');
            // esc -> empty comment box
            this.get('boundingBox').delegate("key", this._empty_comment_box, 'down:27', 'input[name=comment]');
            // y -> deployable
            this.get('boundingBox').delegate("key", Y.bind(this._submit_yes_form, this), 'down:89', '.revision');
            // u -> unknown
            this.get('boundingBox').delegate("key", Y.bind(this._submit_unknown_form, this), 'down:85', '.revision');
            // n -> no
            this.get('boundingBox').delegate("key", Y.bind(this._submit_no_form, this), 'down:78', '.revision');
        },
        _get_change_status_url: function() {
            var data = this.get('data');
            return url = '/api/v1/project/' + data.project.name + '/+revision-by-pk/' + data.pk + '/+change-status';
        },
        _set_loading: function() {
            this.get('boundingBox').one('.status_bar').addClass('loading');
        },
        _submit_deployability_form: function(which) {
            var rev_node = this.get('boundingBox').one('.revision');
            var form = rev_node.one("form.deployability." + which);
            if (form.ancestor(".revision") != rev_node) {
                return
            }
            this._set_loading();
            Y.io(this._get_change_status_url(), this._get_io_cfg(form));
        },
        _has_modifiers: function(e) {
            return (e.altKey || e.ctrlKey || e.metaKey || e.shiftKey);
        },
        _submit_yes_form: function(e) {
            if (e.target.test("input[type=text]")) {
                return;
            }
            if (this._has_modifiers(e)) {
                return;
            }
            this._submit_deployability_form('yes');
            e.preventDefault();
        },
        _submit_unknown_form: function(e) {
            if (e.target.test("input[type=text]")) {
                return;
            }
            if (this._has_modifiers(e)) {
                return;
            }
            this._submit_deployability_form('unknown');
            e.preventDefault();
        },
        _submit_no_form: function(e) {
            if (e.target.test("input[type=text]")) {
                return;
            }
            if (this._has_modifiers(e)) {
                return;
            }
            this._submit_deployability_form('no');
            e.preventDefault();
        },
        _focus_comment_box: function(e) {
            if (e.target.test("input[type=text]")) {
                return;
            }
            if (this._has_modifiers(e)) {
                return;
            }
            if (e.target == this) {
                this.one("input[name=comment]").focus();
            }
            e.preventDefault();
        },
        _empty_comment_box: function(e) {
            if (this._has_modifiers(e)) {
                return;
            }
            e.currentTarget.set('value', '');
            e.currentTarget.ancestor('.revision').focus();
        },
        _get_comment_io_cfg: function(form) {
            // Would be nice to avoid the extra GET caused by the
            // redirect after submitting the comment form.
            io_cfg = this._get_io_cfg(form);
            io_cfg.on.success = Y.bind(this._add_comment_success, this);
            return io_cfg;
        },
        _get_io_cfg: function(form) {
            return {
                method: form.get('method'),
                form: {id: form},
                on: {
                    success: Y.bind(this._update_deployed_success, this),
                    failure: Y.bind(this._update_deployed_failure, this)
                }
            };
        },
        _update_deployed_success: function(id, xhr_response, args) {
            var response = Y.JSON.parse(xhr_response.responseText);
            if (response.status == 'ok') {
                Y.io(this.get('refresh_url'), {
                    method: 'get',
                    on: {
                        success: Y.bind(this._refresh_data, this),
                        failure: Y.bind(this._refresh_failure, this)
                    }
                });
            } else {
                console.log(response);
                alert("Request failed, please try again.");
            }
        },
        _refresh_data: function(id, xhr_response, args) {
            var response = Y.JSON.parse(xhr_response.responseText);
            this.fire('revision:revision_status_updated', {data: response});
        },
        _add_comment_success: function(id, xhr_response, args) {
            this.get('contentBox').one('input[name=comment]').set('value', '');
            Y.io(this.get('refresh_url'), {
                method: 'get',
                on: {
                    success: Y.bind(this._refresh_data, this),
                    failure: Y.bind(this._refresh_failure, this)
                }
            });
        },
        _refresh_failure: function(id, response, args) {
            console.log(response);
            alert("Updating page failed, please try refreshing.");
        },
        _update_deployed_failure: function(id, response, args) {
            console.log(response);
            alert("Request failed, please try again.");
        }
    }, {
        ATTRS: {
            data: {
                value: {}
            },
            refresh_url: {
                value: ''
            }
        },
        HTML_PARSER: {
            data: function(contentBox) {
                return Y.JSON.parse(contentBox.one('.revision').getAttribute('data-all'));
            }
        },
        CONTENT_TEMPLATE: null
    });

}, "0.0.1", { requires: ["base-build", "widget", "widget-parent", "widget-child", "oop", "handlebars", "json-parse", "isitdeployable-templating", "node-focusmanager", "io-form"] });
