YUI.add("isitdeployable-templating", function (Y) {

    var Templating = Y.Base.create("templating", Y.Base, []);

    Templating.get_template_source = function(name) {
        return Y.one('#' + name + '-template').getHTML();
    };
    Templating.compile_template = function(name) {
        return Y.Handlebars.compile(Templating.get_template_source(name));
    };

    Templating.register_partials = function() {
        Y.Handlebars.registerPartial('revision_form', Templating.get_template_source('revision_form'));
        Y.Handlebars.registerPartial('revision_comment_form', Templating.get_template_source('revision_comment_form'));
        Y.Handlebars.registerPartial('revision_info', Templating.get_template_source('revision_info'));
        Y.Handlebars.registerPartial('deployability_form', Templating.get_template_source('deployability_form'));
        Y.Handlebars.registerPartial('editor', Templating.get_template_source('editor'));
    };

    function exclude(options) {
        return '';
    }
    function include(options) {
        return options.fn(this);
    }

    Templating.set_include_project = function(include_project) {
        if (include_project) {
            Y.Handlebars.registerHelper('if_include_project', include);
        } else {
            Y.Handlebars.registerHelper('if_include_project', exclude);
        }
    };
    Templating.set_include_assignee = function(include_assignee) {
        if (include_assignee) {
            Y.Handlebars.registerHelper('if_include_assignee', include);
        } else {
            Y.Handlebars.registerHelper('if_include_assignee', exclude);
        }
    };

    Y.Handlebars.registerHelper('dump_json', function(info) {
        return Y.JSON.stringify(this);
    });

    Y.Handlebars.registerHelper('current_url', function(info) {
        return document.URL;
    });
    Y.Handlebars.registerHelper('linebreaks', function(text) {
        // Remove evil stuff from passed string
        text = Y.Handlebars.Utils.escapeExpression(text);
        // Mimic Django's normalize_newlines.
        text = text.replace(/(\r\n|\r)/gm, '\n');
        // Mimic Django's linebreaks
        pieces = text.split(/\n{2,}/);
        text = "<p>" + pieces.join("</p><p>") + "</p>";
        text = text.replace(/\n/gm, "<br />");
        // Return non-escaped string with raw html
        return new Y.Handlebars.SafeString(text);
    });

    Templating.set_csrf_token = function(csrf_token) {
        Y.Handlebars.registerHelper('csrf_token', function() {
            return csrf_token;
        });
    };

    Templating.set_user = function(user) {
        Y.Handlebars.registerHelper('if_user', function(attribute, options) {
            if (user[attribute]) {
                return options.fn(this);
            } else {
                return options.inverse(this);
            };
        });
        Y.Handlebars.registerHelper('user_info', function() {
            return Y.JSON.stringify(user);
        });
    };

    Templating.set_flags = function(flags) {
        Y.Handlebars.registerHelper('flag', function(flag, options) {
            if (flags[flag]) {
                return options.fn(this);
            } else {
                return options.inverse(this);
            };
        });
        Y.Handlebars.registerHelper('flag_info', function() {
            return Y.JSON.stringify(flags);
        });
    };

    Y.namespace("isitdeployable").Templating = Templating;

}, "0.0.1", { requires: ["base-build", "handlebars", "json-stringify"] });
