from django.conf import settings
from django.contrib.auth.models import (
    AnonymousUser,
    Group,
    User,
    )
from django.test import RequestFactory

from django_factory import TestCase

from ..api_helpers import check_user_allowed


class CheckUserAllowedTests(TestCase):

    def test_user_not_authed(self):
        request = RequestFactory().get('/')
        request.user = AnonymousUser()
        called = object()
        resp = check_user_allowed(lambda r: called)(request)
        self.assertEqual(302, resp.status_code)
        self.assertEqual(settings.LOGIN_URL + '?next=/', resp['location'])

    def test_user_inactive(self):
        request = RequestFactory().get('/')
        request.user = self.factory.make_one(User, is_active=False)
        called = object()
        resp = check_user_allowed(lambda r: called)(request)
        self.assertEqual(403, resp.status_code)

    def test_no_groups(self):
        request = RequestFactory().get('/')
        request.user = self.factory.make_one(User, is_active=True)
        request.user.groups.clear()
        called = object()
        resp = check_user_allowed(lambda r: called)(request)
        self.assertEqual(403, resp.status_code)

    def test_with_groups(self):
        request = RequestFactory().get('/')
        group = self.factory.make_one(Group)
        request.user = self.factory.make_one(User, is_active=True)
        group.user_set.add(request.user)
        called = object()
        resp = check_user_allowed(lambda r: called)(request)
        self.assertEqual(called, resp)
