from django_factory import TestCase
import soupmatchers

from .. import templating


class RevisionCommentForm(soupmatchers.Tag):

    def __init__(self, info, *args, **kwargs):
        kwargs.setdefault('attrs', {})
        kwargs['attrs']['id'] = 'revision_comment_%s_%s' % (info['branch'],
                                                            info['revno'])
        if 'method' not in kwargs['attrs']:
            kwargs['attrs']['method'] = 'post'
        if 'action' not in kwargs['attrs']:
            kwargs['attrs']['action'] = info['comment_form']['url']
        super(RevisionCommentForm, self).__init__(
            'revision %s add comment form' % info['revno'],
            'form',
            *args,
            **kwargs)


class RevisionCommentFormTests(TestCase):

    def render_template(self, info):
        return u"".join(templating.render_template(
            'revtracker/revision_comment_form.html', info))

    def test_has_form(self):
        info = dict(branch="lp:foo", revno="1")
        info['comment_form'] = dict(url='https://example.com/comment_form_url')
        html = self.render_template(info)
        self.assertThat(
            html, soupmatchers.HTMLContains(RevisionCommentForm(info)))


class MarkdownizeTests(TestCase):

    def test_bleach_doesnt_destroy_markdown(self):
        the_markdown = (
            "# header 1\n"
            "\n"
            "<script language='javascript'> alert('hacked')</script>\n"
            " * unordered list item\n"
            "\n"
            "## header 2\n"
            "\n"
            "paragraph\n"
            "\n"
            " 1. ordered list item\n"
            )
        html_text = templating.markdownize(the_markdown)
        self.assertIn('<h1>header 1</h1>', html_text)
        self.assertIn('<ul>\n<li>unordered list item</li>', html_text)
        self.assertIn('<ol>\n<li>ordered list item</li>', html_text)
