#! /bin/bash
# How the tests are run in Jenkins by Tarmac
#
# ./run-tests -- run unit tests for an environment.
#
# Usage: ./run-tests


set -e

BOOTSTRAP=${1:-yes}

# make sure that dependencies are up to date
if [ $BOOTSTRAP = "yes" ]; then
    echo "Bootstrapping..."
    # FIXME: If this runs on the host, the built venv may be broken for
    # containers. This should be run inside the base container.
    make bootstrap
fi

CURRENT_DIR="`pwd`"
# Set up the correct Django configuration.
LOCAL_SETTINGS_PATH="$CURRENT_DIR/../local_config/settings.py"
echo "from django_project.settings_devel import *" > "$LOCAL_SETTINGS_PATH"

function clean_up {
    set +e
    make stop-db
    exit
}

# handle exit cleanly (mainly Ctrl-C)
trap clean_up EXIT SIGHUP SIGINT SIGTERM

echo "Checking for possible conflicting migrations"
DUPLICATES=$(ls **/migrations/*.py | cut -d '_' -f 1 | uniq -d || true)
if [ "${DUPLICATES}" != "" ]; then
    echo "Possible duplicate migrations found:"
    echo ${DUPLICATES}
    exit 1
fi

make start-db

echo "Running isitdeployable tests for tarmac"
# Note that isitdeployable uses Django 1.5 which is dumb and runs tests even
# for wheel dependencies, which we don't want. So we limit it to the apps that
# are actually from this project. Once we migrate to Django 1.6 or newer, this
# can be just ARGS="--failfast"
make test ARGS="--failfast checklist revtracker"
